Attribute VB_Name = "DDEClientModule"
Option Explicit

Dim ServerConnectedFlag(FirstServer To LastServer) As Boolean
Dim ServerRetryFlag(FirstServer To LastServer) As Boolean
Dim ServerDiedFlag(FirstServer To LastServer) As Boolean

Dim InteropCaption As String

Dim LastSearchCallbookCallsign As String

Dim ErrorLogExistsFlag As Boolean

Sub Initialize()
    Dim I As Integer
    
    On Error GoTo Abort
    
    ErrorLogExistsFlag = False
    
    Load DDEClient
    
    For I = FirstServer To LastServer
        ServerConnectedFlag(I) = False
        ServerRetryFlag(I) = False
        ServerDiedFlag(I) = False
    Next I
    
    InteropCaption = ""
    
    ConnectToServer (XcvrControlServer)
    ConnectToServer (DXViewServer)
    ConnectToServer (LogServer)
    ConnectToServer (QSLInfoServer)
    ConnectToServer (PropServer)
    ConnectToServer (SpotServer)
    ConnectToServer (ModeServer)
    
    Exit Sub
    
Abort: Common.ProgramError Err.Number, "DDEClientmodule.Initialize"

End Sub

Sub Terminate()
    Unload DDEClient
End Sub

Function ServerConnected(theServer As Integer) As Boolean
    ServerConnected = ServerConnectedFlag(theServer)
End Function

Sub RetryDeadServers()
    Dim I As Integer
    
    DDEClient.ServerRetryTimer.Enabled = False
    For I = FirstServer To LastServer
        If ServerDiedFlag(I) Then
            ServerDiedFlag(I) = False
            'Debug.Print ServerName(I) + " retrying " + Format$(UTC.CurrentTime, "hh:nn:ss")
            ServerRetryFlag(I) = True
            ConnectToServer I
        End If
    Next I
        
End Sub

Sub SetServerConnected(theServer As Integer, theState As Boolean)
    
    Dim I As Integer
    
    On Error GoTo Abort
    
    ServerConnectedFlag(theServer) = theState
    
    Select Case theServer
    
        Case XcvrControlServer
            DXLabControlMain.UpdateCmd.Enabled = theState
            DXLabControlMain.XcvrMode.Visible = theState
            DXLabControlMain.RadioMode.Visible = theState
            DXLabControlMain.XcvrVFO.Visible = theState
            DXLabControlMain.SecondaryVFO.Visible = theState
            DXLabControlMain.XmitFreq.Visible = theState
            DXLabControlMain.RcvFreq.Visible = theState
            DXLabControlMain.RadioSplit.Visible = theState
            DXLabControlMain.RadioState.Visible = theState
            DXLabControlMain.RadioModel.Visible = theState
            DXLabControlMain.RadioName.Visible = theState
            
            If theState Then
                DXLabControlMain.NewFreq
                DXLabControlMain.NewMode
                DXLabControlMain.NewRadioMode
                DXLabControlMain.NewSplit
            End If
            
        Case LogServer
                        
        Case QSLInfoServer
                        
        Case DXViewServer
        
        Case SpotServer
        
            DXLabControlMain.SpotClusterCmd.Enabled = theState
            DXLabControlMain.SpotLocalCmd.Enabled = theState
            
        Case ModeServer
            
    End Select
    
    InteropCaption = ""
    For I = FirstServer To LastServer
        If ServerConnected(I) Then
            InteropCaption = InteropCaption + ServerNickName(I) + ","
        End If
    Next I
    If Len(InteropCaption) > 0 Then
        InteropCaption = Mid$(InteropCaption, 1, Len(InteropCaption) - 1) 'remove extraneous comma
    End If
        
    If theState Then
        If ServerRetryFlag(theServer) Then
            'Debug.Print ServerName(theServer) + " reconnected " + Format$(UTC.CurrentTime, "hh:nn:ss")
        End If
        
        ServerRetryFlag(theServer) = False
    Else
    
        If ServerRetryFlag(theServer) Then
            ServerRetryFlag(theServer) = False 'give up
            'Debug.Print ServerName(theServer) + " dead " + Format$(UTC.CurrentTime, "hh:nn:ss")
        Else
            ServerDiedFlag(theServer) = True
            'Debug.Print ServerName(theServer) + " died; initiating retry " + Format$(UTC.CurrentTime, "hh:nn:ss")
            On Error Resume Next 'if shutting down, DDEClient is already unloaded
            DDEClient.ServerRetryTimer.Enabled = True 'schedule a retry
            On Error GoTo Abort
        End If
    End If
    
    'Debug.Print ServerName(theServer); " ";
    'If theState Then
    '    Debug.Print "connected"
    'Else
    '    Debug.Print "disconnected"
    'End If
    
    DDEClientModule.UpdateCaption
    
    Exit Sub
    
Abort:
    Common.ProgramError Err.Number, "DDEClientModule.SetServerConnect"
End Sub

Sub ConnectToServer(theServer As Integer)
    
    On Error GoTo ConnectFailed
    
    Select Case theServer
    
        Case XcvrControlServer
        
            DDEClient.Freq.LinkItem = "FreqDisplay"
            DDEClient.Freq.LinkTopic = "CI-V Commander|CIV_CommanderUI"
            DDEClient.Freq.LinkMode = 1 'generates LinkOpen event if successful
            
            DDEClient.Mode.LinkItem = "Mode"
            DDEClient.Mode.LinkTopic = "CI-V Commander|DDEServer"
            DDEClient.Mode.LinkMode = 1 'generates LinkOpen event if successful
            
            DDEClient.RadioMode.LinkItem = "RadioMode"
            DDEClient.RadioMode.LinkTopic = "CI-V Commander|DDEServer"
            DDEClient.RadioMode.LinkMode = 1 'generates LinkOpen event if successful
                       
            DDEClient.DDETransceiverControl.LinkTopic = "CI-V Commander|DDEServer"
            DDEClient.DDETransceiverControl.LinkItem = "DDECommand"
            DDEClient.DDETransceiverControl.LinkMode = 1
            
            SetServerConnected theServer, True
            
            On Error Resume Next 'tolerate down-rev versions of Commander
            
            DXLabControlMain.SMeter.LinkItem = "SMeterValue"
            DXLabControlMain.SMeter.LinkTopic = "CI-V Commander|CIV_CommanderUI"
            DXLabControlMain.SMeter.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.SecondaryVFO.LinkItem = "AltFreqDisplay"
            DXLabControlMain.SecondaryVFO.LinkTopic = "CI-V Commander|CIV_CommanderUI"
            DXLabControlMain.SecondaryVFO.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.RadioState.LinkItem = "RadioState"
            DXLabControlMain.RadioState.LinkTopic = "CI-V Commander|DDEServer"
            DXLabControlMain.RadioState.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.RadioModel.LinkItem = "RadioModel"
            DXLabControlMain.RadioModel.LinkTopic = "CI-V Commander|DDEServer"
            DXLabControlMain.RadioModel.LinkMode = 1 'generates LinkOpen event if successful

            DXLabControlMain.RadioName.LinkItem = "RadioName"
            DXLabControlMain.RadioName.LinkTopic = "CI-V Commander|DDEServer"
            DXLabControlMain.RadioName.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.XmitFreq.LinkItem = "XmitFreq"
            DXLabControlMain.XmitFreq.LinkTopic = "CI-V Commander|DDEServer"
            DXLabControlMain.XmitFreq.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.RcvFreq.LinkItem = "RcvFreq"
            DXLabControlMain.RcvFreq.LinkTopic = "CI-V Commander|DDEServer"
            DXLabControlMain.RcvFreq.LinkMode = 1 'generates LinkOpen event if successful
            
            DDEClient.Split.LinkItem = "RadioSplit"
            DDEClient.Split.LinkTopic = "CI-V Commander|DDEServer"
            DDEClient.Split.LinkMode = 1 'generates LinkOpen event if successful
            DXLabControlMain.NewSplit '1.2.9
            
            DXLabControlMain.SelectedVFO.LinkItem = "RadioVFO"
            DXLabControlMain.SelectedVFO.LinkTopic = "CI-V Commander|DDEServer"
            DXLabControlMain.SelectedVFO.LinkMode = 1
            
            DDEClient.LastUpdate.LinkItem = "RadioLastUpdate"
            DDEClient.LastUpdate.LinkTopic = "CI-V Commander|DDEServer"
            DDEClient.LastUpdate.LinkMode = 1 'generates LinkOpen event if successful
            
        
        Case DXViewServer
        
            DDEClient.SpotPrefix.LinkItem = "SpotPrefix"
            DDEClient.SpotPrefix.LinkTopic = "DXView|DDEServer"
            DDEClient.SpotPrefix.LinkMode = 1 'generates LinkOpen event if successful
                        
            DDEClient.DDELookup.LinkItem = "DDECommand"
            DDEClient.DDELookup.LinkTopic = "DXView|DDEServer"
            DDEClient.DDELookup.LinkMode = 1
            
            
            DXLabControlMain.CurrentLatitude.LinkItem = "SelectedLatitude"
            DXLabControlMain.CurrentLatitude.LinkTopic = "DXView|DXViewMain"
            DXLabControlMain.CurrentLatitude.LinkMode = 1
            
            DXLabControlMain.CurrentLongitude.LinkItem = "SelectedLongitude"
            DXLabControlMain.CurrentLongitude.LinkTopic = "DXView|DXViewMain"
            DXLabControlMain.CurrentLongitude.LinkMode = 1
            
            DXLabControlMain.CurrentGrid.LinkItem = "SelGrid"
            DXLabControlMain.CurrentGrid.LinkTopic = "DXView|DXViewMain"
            DXLabControlMain.CurrentGrid.LinkMode = 1
            
            DXLabControlMain.Location.LinkItem = "Location"
            DXLabControlMain.Location.LinkTopic = "DXView|DXViewMain"
            DXLabControlMain.Location.LinkMode = 1
            
            DXLabControlMain.CQ.LinkItem = "CQ"
            DXLabControlMain.CQ.LinkTopic = "DXView|DXViewMain"
            DXLabControlMain.CQ.LinkMode = 1
            
            DXLabControlMain.ITU.LinkItem = "ITU"
            DXLabControlMain.ITU.LinkTopic = "DXView|DXViewMain"
            DXLabControlMain.ITU.LinkMode = 1
            
            DXLabControlMain.CurrentBearing.LinkItem = "ShortPath"
            DXLabControlMain.CurrentBearing.LinkTopic = "DXView|DXViewMain"
            DXLabControlMain.CurrentBearing.LinkMode = 1
            
            DXLabControlMain.RotatorBearing.LinkItem = "LastRotorHeading"
            DXLabControlMain.RotatorBearing.LinkTopic = "DXView|DXViewMain"
            DXLabControlMain.RotatorBearing.LinkMode = 1
    
            SetServerConnected theServer, True
            
        Case LogServer
        
            DDEClient.DDELog.LinkTopic = "DXKeeper|DDEServer"
            DDEClient.DDELog.LinkItem = "DDECommand"
            DDEClient.DDELog.LinkMode = 1
            
            On Error Resume Next 'run with older versions of DXKeeper
            
            'setup DDE links to lookup results
            
            DXLabControlMain.CallbookName.LinkItem = "CallbookName"
            DXLabControlMain.CallbookName.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.CallbookName.LinkMode = 1
            
            DXLabControlMain.CallbookAddress.LinkItem = "CallbookAddress"
            DXLabControlMain.CallbookAddress.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.CallbookAddress.LinkMode = 1
            
            DXLabControlMain.CallbookCounty.LinkItem = "CallbookCounty"
            DXLabControlMain.CallbookCounty.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.CallbookCounty.LinkMode = 1
            
            DXLabControlMain.CallbookState.LinkItem = "CallbookState"
            DXLabControlMain.CallbookState.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.CallbookState.LinkMode = 1
            
            DXLabControlMain.CallbookProvince.LinkItem = "CallbookProvince"
            DXLabControlMain.CallbookProvince.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.CallbookProvince.LinkMode = 1
            
            DXLabControlMain.CallbookQTH.LinkItem = "CallbookQTH"
            DXLabControlMain.CallbookQTH.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.CallbookQTH.LinkMode = 1
            
            DXLabControlMain.CallbookGrid.LinkItem = "CallbookGrid"
            DXLabControlMain.CallbookGrid.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.CallbookGrid.LinkMode = 1
            
            DXLabControlMain.CallbookError.LinkItem = "CallbookError"
            DXLabControlMain.CallbookError.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.CallbookError.LinkMode = 1
            
            DXLabControlMain.LastQSODateTime.LinkItem = "LastQSODateTime"
            DXLabControlMain.LastQSODateTime.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.LastQSODateTime.LinkMode = 1
            
            DXLabControlMain.LastQSOFreq.LinkItem = "LastQSOFreq"
            DXLabControlMain.LastQSOFreq.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.LastQSOFreq.LinkMode = 1
            
            DXLabControlMain.LastQSOBand.LinkItem = "LastQSOBand"
            DXLabControlMain.LastQSOBand.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.LastQSOBand.LinkMode = 1
            
            DXLabControlMain.LastQSOMode.LinkItem = "LastQSOMode"
            DXLabControlMain.LastQSOMode.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.LastQSOMode.LinkMode = 1
            
            DXLabControlMain.LastQSOQSLRcvd.LinkItem = "LastQSOQSLRcvd"
            DXLabControlMain.LastQSOQSLRcvd.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.LastQSOQSLRcvd.LinkMode = 1
            
            DXLabControlMain.PrevQSOCount.LinkItem = "LastQSOCount"
            DXLabControlMain.PrevQSOCount.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.PrevQSOCount.LinkMode = 1
                        
            DXLabControlMain.CallbookCallsign.LinkItem = "LookupCallsign"
            DXLabControlMain.CallbookCallsign.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.CallbookCallsign.LinkMode = 1
            
            DXLabControlMain.SearchComplete.LinkItem = "SearchComplete"
            DXLabControlMain.SearchComplete.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.SearchComplete.LinkMode = 1
            
            DXLabControlMain.DXKeeperDXCCPrefix.LinkItem = "DXCCPrefix"
            DXLabControlMain.DXKeeperDXCCPrefix.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.DXKeeperDXCCPrefix.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.DXKeeperDXCCID.LinkItem = "DXCCID"
            DXLabControlMain.DXKeeperDXCCID.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.DXKeeperDXCCID.LinkMode = 1 'generates LinkOpen event if successful

            DXLabControlMain.DXKeeperCont.LinkItem = "Cont"
            DXLabControlMain.DXKeeperCont.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.DXKeeperCont.LinkMode = 1 'generates LinkOpen event if successful
        
            DXLabControlMain.DXKeeperCQ.LinkItem = "CQ"
            DXLabControlMain.DXKeeperCQ.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.DXKeeperCQ.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.DXKeeperITU.LinkItem = "ITU"
            DXLabControlMain.DXKeeperITU.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.DXKeeperITU.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.DXKeeperIOTA.LinkItem = "Iota"
            DXLabControlMain.DXKeeperIOTA.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.DXKeeperIOTA.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.DXKeeperLotW.LinkItem = "LotW"
            DXLabControlMain.DXKeeperLotW.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.DXKeeperLotW.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.DXKeepereQSL.LinkItem = "eQSL"
            DXLabControlMain.DXKeepereQSL.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.DXKeepereQSL.LinkMode = 1 'generates LinkOpen event if successful
                        
            DXLabControlMain.DXKeeperQSLVia.LinkItem = "QSLVia"
            DXLabControlMain.DXKeeperQSLVia.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.DXKeeperQSLVia.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.DXKeeperNeeded.LinkItem = "Needed"
            DXLabControlMain.DXKeeperNeeded.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.DXKeeperNeeded.LinkMode = 1 'generates LinkOpen event if successful
            
            DXLabControlMain.CallbookResults.LinkItem = "LookupResults"
            DXLabControlMain.CallbookResults.LinkTopic = "DXKeeper|DDEServer"
            DXLabControlMain.CallbookResults.LinkMode = 1
            
            DXLabControlMain.FilterPanelTextbox.LinkItem = "FilterField"
            DXLabControlMain.FilterPanelTextbox.LinkTopic = "DXKeeper|DXLogMain"
            DXLabControlMain.FilterPanelTextbox.LinkMode = 1
            
            SetServerConnected theServer, True
            
        Case QSLInfoServer
            DDEClient.PathfinderDDECommand.LinkItem = "DDECommand"
            DDEClient.PathfinderDDECommand.LinkTopic = "Pathfinder|DDEServer"
            DDEClient.PathfinderDDECommand.LinkMode = 1
    
            SetServerConnected theServer, True
            
        Case PropServer

            DDEClient.PropViewDDECommand.LinkItem = "DDECommand"
            DDEClient.PropViewDDECommand.LinkTopic = "PropView|DDEServer"
            DDEClient.PropViewDDECommand.LinkMode = 1
    
            SetServerConnected theServer, True
            
        Case SpotServer
            DDEClient.SpotCollectorDDECommand.LinkItem = "DDECommand"
            DDEClient.SpotCollectorDDECommand.LinkTopic = "SpotCollector|DDEServer"
            DDEClient.SpotCollectorDDECommand.LinkMode = 1
    
            SetServerConnected theServer, True
        
            On Error Resume Next 'tolerate older versions of SpotCollector
            
            DXLabControlMain.SpotDatabasePathname.LinkItem = "SpotDatabasePathname"
            DXLabControlMain.SpotDatabasePathname.LinkTopic = "SpotCollector|Config"
            DXLabControlMain.SpotDatabasePathname.LinkMode = 1
            
            DXLabControlMain.LastIndex.LinkItem = "LastSpotDatabaseIndex"
            DXLabControlMain.LastIndex.LinkTopic = "SpotCollector|SpotDatabaseUI"
            DXLabControlMain.LastIndex.LinkMode = 1
        
        Case ModeServer
            DDEClient.WinWarblerDDECommand.LinkItem = "DDECommand"
            DDEClient.WinWarblerDDECommand.LinkTopic = "WinWarbler|DDEServer"
            DDEClient.WinWarblerDDECommand.LinkMode = 1
            
            SetServerConnected theServer, True
            
    End Select
    
    Exit Sub

ConnectFailed:
    SetServerConnected theServer, False
End Sub

Function GetInteropCaption() As String

    GetInteropCaption = InteropCaption
    
End Function

Sub UpdateXcvrFreq(theFreq As Double)

    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000")) + QSYCommand + Format$(theFreq)

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub UpdateXCVRFreqMode(theFreq As String, theMode As String, PreserveDualandSplit As Boolean)

    Dim Preservation As String
    
    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    If PreserveDualandSplit Then
        Preservation = "Y"
    Else
        Preservation = "N"
    End If
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000")) + XCVRFreqModeCommand + DDECommon.EncodeField(XcvrFreq, theFreq) + DDECommon.EncodeField(XcvrMode, theMode) + DDECommon.EncodeField(XcvrPreserveSplitAndDual, Preservation)

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub UpdateXCVRMode(theMode As String)
    
    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000")) + XCVRSetModeCommand + DDECommon.EncodeField(XcvrSetMode, theMode)

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub SetVFOA()

    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000")) + SetVFOACommand

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub SetVFOB()

    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000")) + SetVFOBCommand

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub SetVFOEqual()
    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000")) + SetVFOEqualCommand

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub SwapVFOs()
    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000")) + SetVFOSwapCommand

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub SetSplit(theValue As Boolean)
    
    Dim Arg As String
    
    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    If theValue Then
        Arg = "on"
    Else
        Arg = "off"
    End If
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000")) + SetSplitCommand + Arg

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub SetDual(theValue As Boolean)

    Dim Arg As String
    
    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    If theValue Then
        Arg = "on"
    Else
        Arg = "off"
    End If
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000")) + SetDualCommand + Arg

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub SetFilter(theFilter As String)
    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000")) + SetFilterCommand + theFilter

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub SetTransmit()
    
    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000") + TransmitCommand)

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub SetReceive()
    
    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000") + ReceiveCommand)

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError
End Sub

Sub SelectXcvr(Index As Integer)
    On Error GoTo Failed
    
    If Not ServerConnected(XcvrControlServer) Then Exit Sub
    
    DDEClient.DDETransceiverControl.LinkExecute (Format$(XcvrControlServer, "000") + SelectPrimaryXcvrCommand + Format$(Index))

    Exit Sub
    
Failed: 'nobody home
    DDEClientModule.HandleXcvrControlServerDDEError

End Sub

Sub CheckCallsign(theCallsign As String)

    On Error GoTo Failed
    
    If ServerConnected(LogServer) Then
        DDEClient.DDELog.LinkExecute (Format$(LogServer, "000") + CheckCommand + theCallsign)
    End If
    Exit Sub
    
Failed:
    DDEClientModule.HandleLogServerDDEError
End Sub

Sub LookupCallsign(theCallsign As String)

    On Error GoTo Failed
    
    If ServerConnected(DXViewServer) Then
        DDEClient.DDELookup.LinkExecute (Format$(DXViewServer, "000") + LookupCommand + theCallsign)
    End If
    Exit Sub

Failed:
    DDEClientModule.HandleDXViewDDEError
End Sub

Sub ShowGrid(theGrid As String)
    
    On Error GoTo Failed
    
    If ServerConnected(DXViewServer) Then
        DDEClient.DDELookup.LinkExecute (Format$(DXViewServer, "000") + ShowGridCommand + theGrid)
    End If
    
Failed:
    DDEClientModule.HandleDXViewDDEError
End Sub

Sub SelectPosition(theLatitude As String, theLongitude As String)
    Dim theParams As String
    On Error GoTo Failed
    
    If ServerConnected(DXViewServer) Then
        theParams = ""
        theParams = theParams + DDECommon.EncodeField(SelectPositionLat, theLatitude)
        theParams = theParams + DDECommon.EncodeField(SelectPositionLon, theLongitude)
        DDEClient.DDELookup.LinkExecute (Format$(DXViewServer, "000") + SelectPositionCommand + theParams)
    End If
    Exit Sub

Failed:
    
    DDEClientModule.HandleDXViewDDEError
End Sub

Sub SelectQTH(theLatitude As String, theLongitude As String, theArea As String)
    Dim theParams As String
    On Error GoTo Failed
    
    If ServerConnected(DXViewServer) Then
        theParams = ""
        theParams = theParams + DDECommon.EncodeField(SelectQTHLat, theLatitude)
        theParams = theParams + DDECommon.EncodeField(SelectQTHLon, theLongitude)
        theParams = theParams + DDECommon.EncodeField(SelectQTHArea, theArea)
        DDEClient.DDELookup.LinkExecute (Format$(DXViewServer, "000") + SelectQTHCommand + theParams)
    End If
    Exit Sub

Failed:
    
    DDEClientModule.HandleDXViewDDEError
End Sub

Sub SelectBearing(theBearing As String)
    On Error GoTo Failed
    
    If ServerConnected(DXViewServer) Then
        DDEClient.DDELookup.LinkExecute (Format$(DXViewServer, "000") + SelectBearingCommand + theBearing)
    End If
    Exit Sub

Failed:
    
    DDEClientModule.HandleDXViewDDEError
End Sub

Sub Rotate(SP As Boolean)
    On Error GoTo Failed
    
    If ServerConnected(DXViewServer) Then
        If SP Then
            DDEClient.DDELookup.LinkExecute (Format$(DXViewServer, "000") + RotateSPCommand)
        Else
            DDEClient.DDELookup.LinkExecute (Format$(DXViewServer, "000") + RotateLPCommand)
        End If
    End If
    Exit Sub

Failed:
    
    DDEClientModule.HandleDXViewDDEError
End Sub

Sub PlotPrefix(thePrefix As String, theBand As String, theMode As String, theOrigin As String)
    Dim theParams As String
    On Error GoTo Failed
    
    If ServerConnected(DXViewServer) Then
        theParams = ""
        theParams = theParams + DDECommon.EncodeField(PlotFullPrefix, thePrefix)
        theParams = theParams + DDECommon.EncodeField(PlotFullPrefixBand, theBand)
        theParams = theParams + DDECommon.EncodeField(PlotFullPrefixMode, theMode)
        theParams = theParams + DDECommon.EncodeField(PlotFullPrefixOrigin, theOrigin)
        DDEClient.DDELookup.LinkExecute (Format$(DXViewServer, "000") + PlotFullPrefixCommand + theParams)
    End If
    Exit Sub

Failed:
    
    DDEClientModule.HandleDXViewDDEError
    
End Sub

Sub PlotCallsign(thePrefix As String, theBand As String, theMode As String, theOrigin As String, theCall As String, theFreq As String, theTime As String, theIndex As String)
    Dim theParams As String
    On Error GoTo Failed
    
    If ServerConnected(DXViewServer) Then
        theParams = ""
        theParams = theParams + DDECommon.EncodeField(PlotFullCallPrefix, thePrefix)
        theParams = theParams + DDECommon.EncodeField(PlotFullCallBand, theBand)
        theParams = theParams + DDECommon.EncodeField(PlotFullCallMode, theMode)
        theParams = theParams + DDECommon.EncodeField(PlotFullCallOrigin, theOrigin)
        theParams = theParams + DDECommon.EncodeField(PlotFullCall, theCall)
        theParams = theParams + DDECommon.EncodeField(PlotFullCallFreq, theFreq)
        theParams = theParams + DDECommon.EncodeField(PlotFullCallTime, theTime)
        theParams = theParams + DDECommon.EncodeField(PlotFullCallIndex, theIndex)
        DDEClient.DDELookup.LinkExecute (Format$(DXViewServer, "000") + PlotFullCallCommand + theParams)
    End If
    Exit Sub

Failed:
    
    DDEClientModule.HandleDXViewDDEError
    
End Sub

Sub LogCallsign(theCallsign As String)
    Dim theParams As String
    
    If ServerConnected(LogServer) Then
        theParams = DDECommon.EncodeField(NewCallsign, theCallsign)
        DDEClient.DDELog.LinkExecute (Format$(LogServer, "000") + NewCallsignCommand + theParams)
    End If

End Sub

Sub ClearCapture()

    On Error GoTo Failed
    
    If ServerConnected(LogServer) Then
        DDEClient.DDELog.LinkExecute (Format$(LogServer, "000") + ClearCaptureCommand)
    End If
    Exit Sub

Failed:
    SetServerConnected LogServer, False
End Sub

Sub Log(theLogCommand As String, theRecord As String)

    On Error GoTo Failed
    
    If ServerConnected(LogServer) Then
        DDEClient.DDELog.LinkExecute (Format$(LogServer, "000") + theLogCommand + theRecord)
    End If
    Exit Sub

Failed:
    SetServerConnected LogServer, False
End Sub

Function BooleanParamString(theParam As Boolean) As String

    On Error GoTo Abort
    
    BooleanParamString = ""
    If theParam Then
        BooleanParamString = "Y"
    End If
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "DDEClientModule.BooleanParamString"
    
End Function

Sub ExternalLog(theADIF As String, UploadeQSL As Boolean, UploadLoTW As Boolean, UploadClubLog As Boolean, DeduceMissing As Boolean, QueryCallbook As Boolean, UpdateeQSLMembership As Boolean, UpdateLoTWMembership As Boolean, CheckOverrides As Boolean)
    Dim theParams As String
    
    On Error GoTo Failed
    '1.4.3
    If ServerConnected(LogServer) Then
        theParams = ""
        theParams = theParams + DDECommon.EncodeField(ExternalLogADIF, theADIF)
        theParams = theParams + DDECommon.EncodeField(ExternalLogUploadeQSL, DDEClientModule.BooleanParamString(UploadeQSL))
        theParams = theParams + DDECommon.EncodeField(ExternalLogUploadLoTW, DDEClientModule.BooleanParamString(UploadLoTW))
        theParams = theParams + DDECommon.EncodeField(ExternalLogDeduceMissing, DDEClientModule.BooleanParamString(DeduceMissing))
        theParams = theParams + DDECommon.EncodeField(ExternalLogQueryCallbook, DDEClientModule.BooleanParamString(QueryCallbook))
        theParams = theParams + DDECommon.EncodeField(ExternalLogQueryCallbook, DDEClientModule.BooleanParamString(UpdateeQSLMembership))
        theParams = theParams + DDECommon.EncodeField(ExternalLogQueryCallbook, DDEClientModule.BooleanParamString(UpdateLoTWMembership))
        theParams = theParams + DDECommon.EncodeField(ExternalLogQueryCallbook, DDEClientModule.BooleanParamString(CheckOverrides))
        
        DDEClient.DDELog.LinkExecute (Format$(LogServer, "000") + ExternalLogCommand + theParams)
    End If
    Exit Sub

Failed:
    SetServerConnected LogServer, False

End Sub

Sub Delete(theRecord As String)

    On Error GoTo Failed
    
    '1.3.8
    If ServerConnected(LogServer) Then
        DDEClient.DDELog.LinkExecute (Format$(LogServer, "000") + DeleteQSOCommand + theRecord)
    End If
    Exit Sub

Failed:
    SetServerConnected LogServer, False
End Sub

Sub Import(theFilename As String)

    On Error GoTo Failed
    
    If ServerConnected(LogServer) Then
        DDEClient.DDELog.LinkExecute (Format$(LogServer, "000") + ImportCommand + theFilename)
    End If
    Exit Sub

Failed:
    SetServerConnected LogServer, False
End Sub

Sub SetDefaultQTHID(theQTHID As String)

    On Error GoTo Failed
    
    If ServerConnected(LogServer) Then
        DDEClient.DDELog.LinkExecute (Format$(LogServer, "000") + SelectQTHIDCommand + theQTHID)
    End If
    Exit Sub

Failed:
    SetServerConnected LogServer, False


End Sub

Sub FilterLog(theSQL As String, theCaption As String)

    On Error GoTo Failed
    '1.4.1
    If ServerConnected(LogServer) Then
        DDEClient.DDELog.LinkExecute (Format$(LogServer, "000") + FilterLogCommand + DDECommon.EncodeField(FilterLogExpression, theSQL) + DDECommon.EncodeField(FilterLogCaption, theCaption))
    End If
Failed:
    SetServerConnected LogServer, False
    
End Sub

Sub Check(theCallsign As String)

    On Error GoTo Failed
    
    If ServerConnected(LogServer) Then
        DDEClient.DDELog.LinkExecute (Format$(LogServer, "000") + CheckCommand + theCallsign)
    End If
    Exit Sub

Failed:
    SetServerConnected LogServer, False
End Sub

Sub CallbookLookup(SearchCallbookCallsign As String)
    
    On Error Resume Next 'in case of timeout
       
    DXLabControlMain.CallbookCmd.Enabled = False
    
    LastSearchCallbookCallsign = SearchCallbookCallsign
    
    DXLabControlMain.CallbookCallsign.Text = ""
    DXLabControlMain.SearchComplete = ""
    DXLabControlMain.CallbookError.Text = ""
    
    DXLabControlMain.LastQSODateTime = ""
    DXLabControlMain.LastQSOFreq = ""
    DXLabControlMain.LastQSOBand = ""
    DXLabControlMain.LastQSOMode = ""
    DXLabControlMain.LastQSOQSLRcvd = ""
    DXLabControlMain.PrevQSOCount = ""
    
    If DDEClientModule.ServerConnected(LogServer) Then
        DDEClient.DDELog.LinkTopic = "DXKeeper|DDEServer"
        DDEClient.DDELog.LinkItem = "DDECommand"
        DDEClient.DDELog.LinkMode = 1
        DDEClient.DDELog.LinkExecute (Format$(LogServer, "000") + SearchCallbookCommand + UCase$(SearchCallbookCallsign))
    End If
    
    Exit Sub
    
End Sub

Sub LookupQSLRoute(theCall As String)
    Dim theParams As String
    On Error GoTo Failed
    
    If ServerConnected(QSLInfoServer) Then
        theParams = DDECommon.EncodeField(QSLCallsign, theCall)
        DDEClient.PathfinderDDECommand.LinkExecute (Format$(QSLInfoServer, "000") + GetQSLInfoCommand + theParams)
    End If
    
    Exit Sub
    
Failed:
    DDEClientModule.HandlePathfinderDDEError
End Sub

Sub SetTXPower(thePower As String)
    Dim theParams As String
    On Error GoTo Failed
    
    If ServerConnected(PropServer) Then
        DDEClient.PropViewDDECommand.LinkExecute (Format$(PropServer, "000") + SetPowerCommand + thePower)
    End If
    
    Exit Sub
    
Failed:
    DDEClientModule.HandlePropViewDDEError
End Sub

Sub Predict(PathKind As Integer)

    Dim theParams As String
    Dim theCmd As String
    
    On Error GoTo Failed
    
    If ServerConnected(PropServer) Then
    
        theCmd = Format$(PropServer, "000")
        
        Select Case PathKind
        
            Case 0 'predict with whatever path is selected on PropView's Main window
                theCmd = theCmd + Predictcommand
            
            Case 1 'switch PropView to shortpath and predict
                theCmd = theCmd + PredictSPcommand
                
            Case 2 'switch PropView to longpath and predict
                theCmd = theCmd + PredictLPcommand
        
        End Select
        
        If DXLabControlMain.PropFreqPrediction.Value = vbChecked Then
            theCmd = theCmd + Format$(DXLabControlMain.PropFreq.Text)
        End If
        
        DDEClient.PropViewDDECommand.LinkExecute theCmd
        
    End If
    
    Exit Sub
    
Failed:
    DDEClientModule.HandlePropViewDDEError
End Sub

Sub Spot(Callsign As String, Frequency As String, Mode As String, Submode As String, Kind As Integer, Notes As String)
    Dim theValue As Integer
    
    On Error GoTo Failed
    
    If Len(Callsign) = 0 Then Exit Sub
        
    If Not ServerConnected(SpotServer) Then Exit Sub
    
    DDEClient.SpotCollectorDDECommand.LinkExecute (Format$(SpotServer, "000")) + ConveySpotCommand + DDECommon.EncodeField(ConveySpotCallsign, Callsign) + DDECommon.EncodeField(ConveySpotFreq, Frequency) + DDECommon.EncodeField(ConveySpotMode, Mode) + DDECommon.EncodeField(ConveySpotNotes, Notes) + DDECommon.EncodeField(ConveySpotKind, Format$(Kind)) + DDECommon.EncodeField(ConveySpotSubmode, Submode)
    Exit Sub
    
Failed:
    DDEClientModule.HandleSpotCollectorDDEError
End Sub

Sub ErrorLogExists()
    ErrorLogExistsFlag = True
    DDEClientModule.UpdateCaption
End Sub

Sub UpdateCaption()

    Dim Interop As String
    Dim ErrorInfo As String
    
    If ErrorLogExistsFlag Then
        ErrorInfo = " - see ErrorLog in " + App.Path + "/ErrorLog.txt"
    Else
        ErrorInfo = ""
    End If
    
    Interop = InteropCaption
    If Len(Interop) > 0 Then
        Interop = " {" + Interop + "}"
    End If
    
    DXLabControlMain.Caption = App.ProductName + " " + Common.Version + Interop + ErrorInfo
    
End Sub

Sub HandleWinWarblerDDEError()
    
    SetServerConnected ModeServer, False
    
End Sub

Sub HandleXcvrControlServerDDEError()
       
    SetServerConnected XcvrControlServer, False
    
End Sub

Sub HandleLogServerDDEError()
       
    SetServerConnected LogServer, False
    
End Sub

Sub HandleDXViewDDEError()
       
    SetServerConnected DXViewServer, False
    
End Sub

Sub HandlePathfinderDDEError()
       
    SetServerConnected QSLInfoServer, False
    
End Sub

Sub HandlePropViewDDEError()
       
    SetServerConnected PropServer, False
    
End Sub

Sub HandleSpotCollectorDDEError()
       
    SetServerConnected SpotServer, False
    
End Sub
