Attribute VB_Name = "Registry"
Option Explicit


Global Const HKEY_CLASSES_ROOT = &H80000000
Global Const HKEY_CURRENT_CONFIG = &H80000005
Global Const HKEY_CURRENT_USER = &H80000001
Global Const HKEY_LOCAL_MACHINE = &H80000002
Global Const HKEY_USERS = &H80000003

Const KEY_READ = &H20019  ' ((READ_CONTROL Or KEY_QUERY_VALUE Or
                          ' KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY) And (Not
                          ' SYNCHRONIZE))
Const REG_SZ = 1
Const REG_EXPAND_SZ = 2
Const REG_BINARY = 3
Const REG_DWORD = 4
Const REG_MULTI_SZ = 7

Public Const ERROR_NONE = 0
Public Const ERROR_BADDB = 1
Public Const ERROR_BADKEY = 2
Public Const ERROR_CANTOPEN = 3
Public Const ERROR_CANTREAD = 4
Public Const ERROR_CANTWRITE = 5
Public Const ERROR_OUTOFMEMORY = 6
Public Const ERROR_ARENA_TRASHED = 7
Public Const ERROR_ACCESS_DENIED = 8
Public Const ERROR_INVALID_PARAMETERS = 87
Public Const ERROR_MORE_DATA = 234
Public Const ERROR_NO_MORE_ITEMS = 259
                          
Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" _
    (ByVal hKey As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, _
    ByVal samDesired As Long, phkResult As Long) As Long

Declare Function RegCloseKey Lib "advapi32.dll" (ByVal hKey As Long) As _
    Long

Declare Function RegEnumKey Lib "advapi32.dll" Alias "RegEnumKeyA" _
    (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, _
    ByVal cbName As Long) As Long

Declare Function RegQueryValueEx Lib "advapi32.dll" Alias _
    "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, _
    ByVal lpReserved As Long, lpType As Long, lpData As Any, _
    lpcbData As Long) As Long

Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (dest As _
    Any, source As Any, ByVal numBytes As Long)

Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" _
    (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpValueName As String, _
    lpcbValueName As Long, ByVal lpReserved As Long, lpType As Long, _
    lpData As Any, lpcbData As Long) As Long
    
Function KeyPresent(ByVal hKey As Long, ByVal KeyName As String) As Boolean

    Dim Handle As Long
    
    KeyPresent = False
    If Len(KeyName) > 0 Then
        If RegOpenKeyEx(hKey, KeyName, 0, KEY_READ, Handle) Then Exit Function
    End If
    KeyPresent = True
    
End Function

' Enumerate registry keys under a given key
' returns a collection of strings
Function EnumRegistryKeys(ByVal hKey As Long, ByVal KeyName As String) As _
    Collection
    Dim Handle As Long
    Dim Length As Long
    Dim Index As Long
    Dim SubkeyName As String
    
    ' initialize the result collection
    Set EnumRegistryKeys = New Collection
    
    ' Open the key, exit if not found
    If Len(KeyName) Then
        If RegOpenKeyEx(hKey, KeyName, 0, KEY_READ, Handle) Then Exit Function
        ' in all case the subsequent functions use hKey
        hKey = Handle
    End If
    
    Do
        ' this is the max length for a key name
        Length = 260
        SubkeyName = Space$(Length)
        ' get the N-th key, exit the loop if not found
        If RegEnumKey(hKey, Index, SubkeyName, Length) Then Exit Do
        
        ' add to the result collection
        SubkeyName = Left$(SubkeyName, InStr(SubkeyName, vbNullChar) - 1)
        EnumRegistryKeys.Add SubkeyName, SubkeyName
        ' prepare to query for next key
        Index = Index + 1
    Loop
   
    ' Close the key, if it was actually opened
    If Handle Then RegCloseKey Handle
        
End Function

'Enumerate values under a given registry key.
'Returns a collection, where each element of the collection is a 3-element array
'of Variants: element(0) is the value name, element(1) is the value's value,
'  element(2) is the type of data type

Function EnumRegistryValuesEx(ByVal hKey As Long, ByVal KeyName As String) As _
    Collection
    Dim Handle As Long
    Dim Index As Long
    Dim valueType As Long
    Dim Name As String
    Dim nameLen As Long
    Dim resLong As Long
    Dim resString As String
    Dim DataLen As Long
    Dim valueInfo(0 To 2) As Variant
    Dim retVal As Long
    
    ' initialize the result
    Set EnumRegistryValuesEx = New Collection
    
    ' Open the key, exit if not found.
    If Len(KeyName) Then
        If RegOpenKeyEx(hKey, KeyName, 0, KEY_READ, Handle) Then Exit Function
        ' in all cases, subsequent functions use hKey
        hKey = Handle
    End If
    
    Do
        ' this is the max length for a key name
        nameLen = 260
        Name = Space$(nameLen)
        ' prepare the receiving buffer for the value
        DataLen = 4096
        ReDim resbinary(0 To DataLen - 1) As Byte
        
        ' read the value's name and data
        ' exit the loop if not found
        retVal = RegEnumValue(hKey, Index, Name, nameLen, ByVal 0&, valueType, _
            resbinary(0), DataLen)
        
        ' enlarge the buffer if you need more space
        If retVal = ERROR_MORE_DATA Then
            ReDim resbinary(0 To DataLen - 1) As Byte
            retVal = RegEnumValue(hKey, Index, Name, nameLen, ByVal 0&, _
                valueType, resbinary(0), DataLen)
        End If
        ' exit the loop if any other error (typically, no more values)
        If retVal Then Exit Do
        
        ' retrieve the value's name
        valueInfo(0) = Left$(Name, nameLen)
        
        ' return a value corresponding to the value type
        Select Case valueType
            Case REG_DWORD
                CopyMemory resLong, resbinary(0), 4
                valueInfo(1) = resLong
                valueInfo(2) = vbLong
            Case REG_SZ, REG_EXPAND_SZ
                ' copy everything but the trailing null char
                resString = Space$(DataLen - 1)
                CopyMemory ByVal resString, resbinary(0), DataLen - 1
                valueInfo(1) = resString
                valueInfo(2) = vbString
            Case REG_BINARY
                ' shrink the buffer if necessary
                If DataLen < UBound(resbinary) + 1 Then
                    ReDim Preserve resbinary(0 To DataLen - 1) As Byte
                End If
                valueInfo(1) = resbinary()
                valueInfo(2) = vbArray + vbByte
            Case REG_MULTI_SZ
                ' copy everything but the 2 trailing null chars
                resString = Space$(DataLen - 2)
                CopyMemory ByVal resString, resbinary(0), DataLen - 2
                valueInfo(1) = resString
                valueInfo(2) = vbString
            Case Else
                ' Unsupported value type - do nothing
        End Select
        
        ' add the array to the result collection
        ' the element's key is the value's name
        EnumRegistryValuesEx.Add valueInfo, valueInfo(0)
        
        Index = Index + 1
    Loop
   
    ' Close the key, if it was actually opened
    If Handle Then RegCloseKey Handle
        
End Function

Function KeyValuePresent(ByVal hKey As Long, KeyName As String, ValueName As String) As Boolean
    
    Dim Handle As Long
    Dim Index As Long
    Dim valueType As Long
    Dim Name As String
    Dim nameLen As Long
    Dim DataLen As Long

    Dim retVal As Long
    
    On Error GoTo Abort
    
    KeyValuePresent = False
    
    ' Open the key, exit if not found.
    If Len(KeyName) Then
        If RegOpenKeyEx(hKey, KeyName, 0, KEY_READ, Handle) Then Exit Function
        ' in all cases, subsequent functions use hKey
        hKey = Handle
    End If
    
    Do
        ' this is the max length for a key name
        nameLen = 260
        Name = Space$(nameLen)
        ' prepare the receiving buffer for the value
        DataLen = 4096
        ReDim resbinary(0 To DataLen - 1) As Byte
        
        ' read the value's name and data
        ' exit the loop if not found
        retVal = RegEnumValue(hKey, Index, Name, nameLen, ByVal 0&, valueType, resbinary(0), DataLen)
        
        ' enlarge the buffer if you need more space
        If retVal = ERROR_MORE_DATA Then
            ReDim resbinary(0 To DataLen - 1) As Byte
            retVal = RegEnumValue(hKey, Index, Name, nameLen, ByVal 0&, valueType, resbinary(0), DataLen)
        End If
        ' exit the loop if any other error (typically, no more values)
        
        If InStr(Name, ValueName) > 0 Then
            KeyValuePresent = True
            Exit Do
        End If
        
        If retVal Then Exit Do
        
        Index = Index + 1
    Loop
   
    ' Close the key, if it was actually opened
    If Handle Then RegCloseKey Handle
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "PathfinderModule.KeyValuePresent"
End Function

Function GetKeyValue(ByVal hKey As Long, KeyName As String, ValueName As String) As String
    
    Dim Handle As Long
    Dim Index As Long
    Dim valueType As Long
    Dim Name As String
    Dim nameLen As Long
    Dim resLong As Long
    Dim resString As String
    Dim DataLen As Long
    Dim valueInfo(0 To 2) As Variant
    Dim retVal As Long
    
    On Error GoTo Abort
    
    GetKeyValue = ""
    
    ' Open the key, exit if not found.
    If Len(KeyName) Then
        If RegOpenKeyEx(hKey, KeyName, 0, KEY_READ, Handle) Then Exit Function
        ' in all cases, subsequent functions use hKey
        hKey = Handle
    End If
    
    Do
        ' this is the max length for a key name
        nameLen = 260
        Name = Space$(nameLen)
        ' prepare the receiving buffer for the value
        DataLen = 4096
        ReDim resbinary(0 To DataLen - 1) As Byte
        
        ' read the value's name and data
        ' exit the loop if not found
        retVal = RegEnumValue(hKey, Index, Name, nameLen, ByVal 0&, valueType, resbinary(0), DataLen)
        
        ' enlarge the buffer if you need more space
        If retVal = ERROR_MORE_DATA Then
            ReDim resbinary(0 To DataLen - 1) As Byte
            retVal = RegEnumValue(hKey, Index, Name, nameLen, ByVal 0&, valueType, resbinary(0), DataLen)
        End If
        ' exit the loop if any other error (typically, no more values)
        
        If InStr(Name, ValueName) > 0 Then

            ' retrieve the value's name
            valueInfo(0) = Left$(Name, nameLen)
            
            ' return a value corresponding to the value type
            Select Case valueType
                Case REG_DWORD
                    CopyMemory resLong, resbinary(0), 4
                    valueInfo(1) = resLong
                    valueInfo(2) = vbLong
                    GetKeyValue = Format$(resLong)
                    
                Case REG_SZ, REG_EXPAND_SZ
                    ' copy everything but the trailing null char
                    resString = Space$(DataLen - 1)
                    CopyMemory ByVal resString, resbinary(0), DataLen - 1
                    valueInfo(1) = resString
                    valueInfo(2) = vbString
                    GetKeyValue = resString
                    
                Case REG_BINARY
                    ' shrink the buffer if necessary
                    If DataLen < UBound(resbinary) + 1 Then
                        ReDim Preserve resbinary(0 To DataLen - 1) As Byte
                    End If
                    valueInfo(1) = resbinary()
                    valueInfo(2) = vbArray + vbByte
                    GetKeyValue = Format$(resbinary())
                    
                Case REG_MULTI_SZ
                    ' copy everything but the 2 trailing null chars
                    resString = Space$(DataLen - 2)
                    CopyMemory ByVal resString, resbinary(0), DataLen - 2
                    valueInfo(1) = resString
                    valueInfo(2) = vbString
                    GetKeyValue = resString
                    
                Case Else
                    ' Unsupported value type - do nothing
            End Select


            Exit Do
        End If
        
        If retVal Then Exit Do
        
        Index = Index + 1
    Loop
   
    ' Close the key, if it was actually opened
    If Handle Then RegCloseKey Handle
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "PathfinderModule.GetKeyValue"
End Function


'Save the specified registry's key and (optionally) its subkeys to a REG file
' that can be loaded later
' - hKey is the root key
' - sKeyName is the key to save to the file
' - sRegFile is the target file where the text will be saved
' - bIncludeSubKeys specifies whether the routine will save also the subkeys
' - bAppendToFile specifies wheter the generated text will be appended to an
' existent file
'Example:
'  SaveRegToFile HKEY_CURRENT_USER, "Software\Microsoft\Visual Basic\6.0",
'  "C:\vb6.reg"

'NOTE: this routine requires EnumRegistryKeys and EnumRegistryValuesEx

Sub SaveRegToFile(ByVal hKey As Long, ByVal sKeyname As String, _
    ByVal sRegFile As String, Optional ByVal bIncludeSubKeys As Boolean = True, _
    Optional ByVal bAppendToFile As Boolean = False)
    
    Dim Handle As Integer
    Dim sFirstKeyPart As String
    Dim col As New Collection
    Dim regItem As Variant
    Dim sText As String
    Dim sQuote As String
    Dim sTemp As String
    Dim sHex As String
    Dim I As Long
    Dim vValue As Variant
    Dim iPointer As MousePointerConstants
    Dim sValueName As String
    
    Dim EmbeddedPathPos As Integer
    Dim ScanPathPos As Integer
    Dim EndPathPos As Integer
    Dim EmbeddedPath As String
    
    sQuote = Chr$(34)
    
    On Error Resume Next
     
    'conver the hKey value to the descriptive string
    Select Case hKey
        Case HKEY_CLASSES_ROOT: sFirstKeyPart = "HKEY_CLASSES_ROOT\"
        Case HKEY_CURRENT_CONFIG: sFirstKeyPart = "HKEY_CURRENT_CONFIG\"
        Case HKEY_CURRENT_USER: sFirstKeyPart = "HKEY_CURRENT_USER\"
        Case HKEY_LOCAL_MACHINE: sFirstKeyPart = "HKEY_LOCAL_MACHINE\"
        Case HKEY_USERS: sFirstKeyPart = "HKEY_USERS\"
    End Select
    
    'this can be a long operation
    iPointer = Screen.MousePointer
    Screen.MousePointer = vbHourglass
    
    'if the text won't be appended, add the "REGEDIT4" header
    If bAppendToFile = False Then
        sText = "REGEDIT4" & vbCrLf & vbCrLf
    Else
        'add the same header if the text will be appended to an
        'existent file that does not contain the header.
        ' This works only if the file exists but is empty.
        Handle = FreeFile
        Open sRegFile For Binary As #Handle
        ' read the string and close the file
        sTemp = Space$(LOF(Handle))
        Get #Handle, , sTemp
        Close #Handle
        'if not found, add it
        If InStr(1, sTemp, "REGEDIT4") = 0 Then
            sText = "REGEDIT4" & vbCrLf & vbCrLf
        End If
    End If
    
    'save the key name with the format [keyname]
    sText = sText & "[" & sFirstKeyPart & sKeyname & "]" & vbCrLf
    
    'get the collection with all the values under this key
    Set col = EnumRegistryValuesEx(hKey, sKeyname)
    For Each regItem In col
        vValue = regItem(1)
        Select Case regItem(2)
            Case vbString
                'if the value is a string, check if it's a path by looking if
                ' the 3 characters
                'are in the form X:\. If so, replace a single "\" with "\\"
                If Left$(vValue, 3) Like "[A-Z,a-z]:\" Then
                    vValue = Replace(vValue, "\", "\\") 'quote it
                Else
                    '1.7.1 deal with pathnames in macro arguments
                    ScanPathPos = 1
                    Do
                        If ScanPathPos >= Len(vValue) Then Exit Do
                        
                        EmbeddedPathPos = InStr(ScanPathPos, vValue, ":\")
                        If EmbeddedPathPos = 0 Then Exit Do
                        
                        If EmbeddedPathPos = 1 Then
                            ScanPathPos = 3
                        ElseIf Mid$(vValue, EmbeddedPathPos - 1, 3) Like "[A-Z,a-z]:\" Then
                            EndPathPos = InStr(EmbeddedPathPos + 3, vValue, ".")
                            If EndPathPos = 0 Then
                                ScanPathPos = EmbeddedPathPos + 3
                            Else
                                
                                Common.DebugLog "Registry.SaveRegToFile: value = " + vValue
                                
                                EmbeddedPath = Mid$(vValue, EmbeddedPathPos, EndPathPos - EmbeddedPathPos + 1)
                                EmbeddedPath = Replace(EmbeddedPath, "\", "\\") 'quote it
                                
                                vValue = Mid$(vValue, 1, EmbeddedPathPos - 1) + EmbeddedPath + Mid$(vValue, EndPathPos + 1, Len(vValue) - EndPathPos)
                                Common.DebugLog "Registry.SaveRegToFile: quoted value = " + vValue
                                
                                ScanPathPos = EndPathPos + 1
                            End If
                        Else
                            ScanPathPos = EmbeddedPathPos + 3
                        End If
                        
                    Loop
                    
                End If
                
                '1.6.9
                If InStr(vValue, """") > 0 Then
                    vValue = Replace(vValue, """", "\""") 'replace
                End If
                
                sTemp = sQuote & vValue & sQuote
            Case vbLong
                'if it's a long, save it with the format dword:num
                sTemp = "dword:" & CLng(vValue)
            Case vbArray + vbByte
                'if it's an array of bytes, save it with the format hex:num1,
                ' num2,num3,...
                sTemp = "hex:"
                For I = 0 To UBound(vValue)
                    sHex = Hex$(vValue(I))
                    'convert from long to hex
                    If Len(sHex) < 2 Then sHex = "0" & sHex
                    sTemp = sTemp & sHex & ","
                Next
                'remove the last comma
                sTemp = Left$(sTemp, Len(sTemp) - 1)
            Case Else
                sTemp = ""
        End Select
        'get the value name: if the string is empty, take @,
        '  else take that name and quote it
        sValueName = IIf(Len(regItem(0)) > 0, sQuote & regItem(0) & sQuote, "@")
        'save this line to the temporary text that will be saved
        sText = sText & sValueName & "=" & sTemp & vbCrLf
    Next
    sText = sText & vbCrLf
    
    Handle = FreeFile
    'open the target file with Append or Output mode,
    '  according to the bAppendToFile parameter
    If bAppendToFile Then
        Open sRegFile For Append As #Handle
    Else
        Open sRegFile For Output As #Handle
    End If
    'save the text
    Print #Handle, sText;
    Close #Handle
    
    'call recursively this routine to save all the subkeys,
    '  if the bIncludeSubKeys param is true
    If bIncludeSubKeys Then
        Set col = EnumRegistryKeys(hKey, sKeyname)
        For Each regItem In col
            'note: the text will be added to the file just created for the
            'values in the root key
            SaveRegToFile hKey, sKeyname & "\" & regItem, sRegFile, True, True
        Next
    End If
    
    Screen.MousePointer = iPointer
    
End Sub

Sub ApplyRegFile(ByVal sRegFile As String)
    On Error Resume Next
    'first of all, check if the file exists
    If Not (GetAttr(sRegFile) And vbDirectory) = 0 Then Exit Sub
    'load the reg file
    '  quote the file name: this is necessary if the file name is something
    ' like "token1 token2.reg"
    Shell "regedit /s " & Chr$(34) & sRegFile & Chr$(34)
End Sub

