Attribute VB_Name = "SocketOptionModule"
Option Explicit


' Error returned by Winsock API.
Global Const SOCKET_ERROR = -1

' Level number for (get/set)sockopt() to apply to socket itself.
Global Const SOL_SOCKET = 65535      ' Options for socket level.
Global Const IPPROTO_TCP = 6         ' Protocol constant for TCP.
Global Const IPPROTO_UDP = 17        ' Protocol constant for UDP.

' option flags per socket
Global Const SO_DEBUG = &H1&         ' Turn on debugging info recording
Global Const SO_ACCEPTCONN = &H2&    ' Socket has had listen() - READ-ONLY.
Global Const SO_REUSEADDR = &H4&     ' Allow local address reuse.
Global Const SO_KEEPALIVE = &H8&     ' Keep connections alive.
Global Const SO_DONTROUTE = &H10&    ' Just use interface addresses.
Global Const SO_BROADCAST = &H20&    ' Permit sending of broadcast msgs.
Global Const SO_USELOOPBACK = &H40&  ' Bypass hardware when possible.
Global Const SO_LINGER = &H80&       ' Linger on close if data present.
Global Const SO_OOBINLINE = &H100&   ' Leave received OOB data in line.

Global Const SO_DONTLINGER = Not SO_LINGER
Global Const SO_EXCLUSIVEADDRUSE = Not SO_REUSEADDR ' Disallow local address reuse.

' Additional options.
Global Const SO_SNDBUF = &H1001&     ' Send buffer size.
Global Const SO_RCVBUF = &H1002&     ' Receive buffer size.
Global Const SO_ERROR = &H1007&      ' Get error status and clear.
Global Const SO_TYPE = &H1008&       ' Get socket type - READ-ONLY.

' TCP Options
Global Const TCP_NODELAY = &H1&      ' Turn off Nagel Algorithm.

' linger structure
Private Type LINGER_STRUCT
  l_onoff As Integer          ' Is linger on or off?
  l_linger As Integer         ' Linger timeout in seconds.
End Type
 
' Winsock API declares
Declare Function setsockopt Lib "wsock32.dll" (ByVal s As Long, ByVal level As Long, ByVal optname As Long, optval As Any, ByVal optlen As Long) As Long
Declare Function getsockopt Lib "wsock32.dll" (ByVal s As Long, ByVal level As Long, ByVal optname As Long, optval As Any, optlen As Long) As Long

Public Function GetSocketOption(lSocket As Long, lLevel As Long, lOption As Long) As Long
  
  On Error GoTo Abort
  
  Dim lresult As Long       ' Result of API call.
  Dim lBuffer As Long       ' Buffer to get value into.
  Dim lBufferLen As Long    ' len of buffer.
  Dim linger As LINGER_STRUCT
  
  ' Linger requires a structure so we will get that option differently.
  If (lOption <> SO_LINGER) And (lOption <> SO_DONTLINGER) Then
    lBufferLen = LenB(lBuffer)
    lresult = getsockopt(lSocket, lLevel, lOption, lBuffer, lBufferLen)
  Else
    lBufferLen = LenB(linger)
    lresult = getsockopt(lSocket, lLevel, lOption, linger, lBufferLen)
    lBuffer = linger.l_onoff
  End If
  
  If (lresult = SOCKET_ERROR) Then
    GetSocketOption = Err.LastDllError
  Else
    GetSocketOption = lBuffer
  End If
  Exit Function
  
Abort:
    Common.ProgramError Err.Number, "SocketOptionsModule.GetSocketOption"
  
End Function

Function GetSocketStatus(theWinsock As Control) As String
  
  On Error GoTo Abort
  
  Dim Socket As Long
  
  Socket = theWinsock.SocketHandle
  
  If Socket = 0 Then
    GetSocketStatus = "no socket"
  Else
    GetSocketStatus = "Socket Options:" & vbCrLf & _
           "    SO_DEBUG: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_DEBUG)) & vbCrLf & _
           "    SO_ACCEPTCONN: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_ACCEPTCONN)) & vbCrLf & _
           "    SO_REUSEADDR: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_REUSEADDR)) & vbCrLf & _
           "    SO_KEEPALIVE: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_KEEPALIVE)) & vbCrLf & _
           "    SO_DONTROUTE: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_DONTROUTE)) & vbCrLf & _
           "    SO_BROADCAST: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_BROADCAST)) & vbCrLf & _
           "    SO_USELOOPBACK: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_USELOOPBACK)) & vbCrLf & _
           "    SO_LINGER: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_LINGER)) & vbCrLf & _
           "    SO_OOBINLINE: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_OOBINLINE)) & vbCrLf & _
           "    SO_DONTLINGER: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_DONTLINGER)) & vbCrLf & _
           "    SO_EXCLUSIVEADDRUSE: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_EXCLUSIVEADDRUSE)) & vbCrLf & _
           "    SO_SNDBUF: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_SNDBUF)) & vbCrLf & _
           "    SO_RCVBUF: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_RCVBUF)) & vbCrLf & _
           "    SO_ERROR: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_ERROR)) & vbCrLf & _
           "    SO_TYPE: " & CStr(GetSocketOption(Socket, SOL_SOCKET, SO_TYPE)) & vbCrLf & vbCrLf & _
           "TCP Options:" & vbCrLf & _
           "    TCP_NODELAY: " & CStr(GetSocketOption(Socket, IPPROTO_TCP, TCP_NODELAY))
    End If
    
    Exit Function
    
Abort:
    Common.ProgramError Err.Number, "SocketOptionsModule.GetSocketStatus"
  
End Function

