Attribute VB_Name = "TCPClientModule"
Option Explicit

Global Const TCPServerSection = "TCPSection"
Global Const TCPServerAddressKey = "TCPServerAddress"
Global Const TCPServerBasePortKey = "TCPServerBasePort"
Global Const WinsockRegisteredKey = "WinsockRegistered"

Dim WinSockRegistered As Boolean

Dim CurrentBase As Long

Dim SendCommandInProgressFlag(MaxTCPOffset) As Boolean
Dim SendCompletedFlag(MaxTCPOffset) As Boolean

Dim LastTCPError As String

Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Public Declare Function RegisterWinsock Lib "mswinsck.ocx" Alias "DllRegisterServer" () As Long


Sub Initialize()
    
    Dim I As Integer
    
    On Error GoTo Abort
    
    For I = 0 To MaxTCPOffset
        SendCommandInProgressFlag(I) = False
    Next I
    
    Load TCPClient
        
    TCPClient.TCPServerAddress.Text = GetSetting(App.ProductName, TCPServerSection, TCPServerAddressKey, DefaultTCPAddress)
    TCPClient.TCPServerPort.Text = GetSetting(App.ProductName, TCPServerSection, TCPServerBasePortKey, DefaultTCPBasePort)
    
    CurrentBase = -1
    On Error Resume Next
    CurrentBase = CLng(TCPClient.TCPServerPort.Text)
    On Error GoTo Abort
    
    TCPClientModule.CheckWinSock
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.Initialize"
    
End Sub


Sub CheckWinSock()
    On Error GoTo Abort
    
    WinSockRegistered = False
    
    If Not CBool(GetSetting(App.ProductName, TCPServerSection, WinsockRegisteredKey, Format$(vbFalse))) Then
        On Error Resume Next
        RegisterWinsock
        If Err.Number > 0 Then
            Common.Log "TCPServerModule.CheckWinSock: unable to register MSWINSCK.OCX: " + Err.Description
        Else
            WinSockRegistered = True
            SaveSetting App.ProductName, TCPServerSection, WinsockRegisteredKey, Format$(vbTrue)
        End If
        On Error GoTo 0
    Else
        WinSockRegistered = True
    End If
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.SetBasePort"
    
End Sub


Sub SetBasePort(thePort As Long)

    On Error GoTo Abort
    
    CurrentBase = thePort
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.SetBasePort"
    
End Sub

Sub SocketError(Index As Integer, Number As Integer, Description As String)

    On Error GoTo Abort
    
    If SendCommandInProgressFlag(Index) Then
        SendCompletedFlag(Index) = True
        SendCommandInProgressFlag(Index) = False
        LastTCPError = "Index " + Format$(Index) + ": " + Description
    End If
    
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.SocketError"
    
End Sub

Function SendCommand(RemoteHost As String, ServerOffset As Integer, Command As String, Parameters As String, Optional StayConnected As Boolean = False) As String
    
    Dim theMessage As String
    
    Dim SleepCount As Long
    
    On Error GoTo Abort
    
    SendCommand = ""
    Common.DebugLog "TCPCommon.SendCommand, Command = " + Command + ", Parameters = " + Parameters
    
    If Len(Trim$(RemoteHost)) = 0 Then
        SendCommand = "error: host address not specified"
        Exit Function
    End If
    
    If CurrentBase = -1 Then
        SendCommand = "error: base port not specified"
        Exit Function
    End If

    If SendCommandInProgressFlag(ServerOffset) Then
        SendCommand = "busy"
        Exit Function
    End If
    
    SendCommandInProgressFlag(ServerOffset) = True
    
    If TCPClient.TCPSocket(ServerOffset).State = 0 Then '1.3.1
        TCPClient.TCPSocket(ServerOffset).Protocol = sckTCPProtocol
        TCPClient.TCPSocket(ServerOffset).RemoteHost = RemoteHost
        TCPClient.TCPSocket(ServerOffset).RemotePort = CurrentBase + ServerOffset
    
        TCPClient.TCPSocket(ServerOffset).Connect
    End If
    
    SleepCount = 0
    Do
        If TCPClient.TCPSocket(ServerOffset).State = sckConnected Then
            Exit Do
    
        ElseIf TCPClient.TCPSocket(ServerOffset).State = sckError Then
            SendCommand = "Error: " + LastTCPError
            TCPClient.TCPSocket(ServerOffset).Close
            SendCommandInProgressFlag(ServerOffset) = False
            Exit Function
        End If
    
        DoEvents
        Sleep (1)
        SleepCount = SleepCount + 1
        If SleepCount > 2000 Then Exit Do
    Loop
    
    If TCPClient.TCPSocket(ServerOffset).State <> sckConnected Then
        SendCommand = "Error: unable to connect"
        TCPClient.TCPSocket(ServerOffset).Close
        SendCommandInProgressFlag(ServerOffset) = False
        Exit Function
    End If
    
    'Debug.Print "Connected " + Format$(SleepCount)
    
    SendCompletedFlag(ServerOffset) = False
    
    theMessage = "<command:" + Format$(Len(Command)) + ">" + Command + "<parameters:" + Format$(Len(Parameters)) + ">" + Parameters
    TCPClient.TCPSocket(ServerOffset).SendData theMessage
    
    SleepCount = 0
    Do
        If SendCompletedFlag(ServerOffset) Then Exit Do
        DoEvents
        Sleep (1)
        SleepCount = SleepCount + 1
        If SleepCount > 2000 Then Exit Do
    Loop
    
    If Not StayConnected Then
        TCPClient.TCPSocket(ServerOffset).Close
    End If
    
    If SendCompletedFlag(ServerOffset) Then
        'Debug.Print "Sent " + Format$(SleepCount)
        SendCommand = "sent"
    Else
        'Debug.Print "Not sent"
        SendCommand = "Error: unable to send data"
        
    End If
    SendCommandInProgressFlag(ServerOffset) = False
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.SendCommand"
    SendCommandInProgressFlag(ServerOffset) = False
    SendCommand = Err.Description
    
End Function

Sub SendComplete(ServerOffset As Integer)

    On Error GoTo Abort
    
    SendCompletedFlag(ServerOffset) = True
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.SendComplete"
    
End Sub

Sub DataArrival(ServerOffset As Integer, ByteCount As Long)

    Dim theRecord As String
    Dim theParseResult As TCPCommon.TCPParseResult
    
    On Error GoTo Abort
    
    TCPClient.TCPSocket(ServerOffset).GetData theRecord, vbString
    
    theParseResult = TCPCommon.ParseParameters(theRecord)
    
    'Debug.Print theRecord
    
    Select Case ServerOffset
    
        Case CommanderTCPOffset
            If theParseResult.ParameterCount = 1 Then
            
                Select Case LCase$(theParseResult.ParameterArray(1).Name)
                
                    Case LCase$(FreqResp)
                        
                        DXLabControlMain.TCPXcvrVFO.Caption = theParseResult.ParameterArray(1).Content

                    Case LCase$(TXFreqResp)
                        
                        DXLabControlMain.TCPXcvrTXVFO.Caption = theParseResult.ParameterArray(1).Content
                     
                    Case LCase$(ModeResp)

                        DXLabControlMain.TCPXcvrRadioMode.Caption = theParseResult.ParameterArray(1).Content
                        
                    Case LCase$(SplitResp)

                        DXLabControlMain.TCPXcvrSplit.Caption = theParseResult.ParameterArray(1).Content
                        
                    Case LCase$(TXResp)
                        DXLabControlMain.TCPXcvrTX.Caption = theParseResult.ParameterArray(1).Content

                End Select
            
            End If
            
    End Select
    
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.DataArrived"
    
End Sub

Function Log(theLogCommand As String, theRecord As String) As Boolean
    
    Dim theResult As String
    
    On Error GoTo Abort
    
    Log = False
    
    If Not DXLabControlMain.LogTCPCmd.Enabled Then Exit Function
    
    DXLabControlMain.LogTCPCmd.Enabled = False

    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, DXKeeperTCPOffset, theLogCommand, theRecord)
    Select Case LCase$(theResult)
    
        Case "sent"
            Log = True
            
        Case "busy"
            Log = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "Log via TCP/IP"
            Log = False
            
    End Select

    DXLabControlMain.LogTCPCmd.Enabled = True
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.Log"
    
End Function

Function ExternalLog(theADIF As String, UploadeQSL As Boolean, UploadLoTW As Boolean, UploadClubLog As Boolean, DeduceMissing As Boolean, QueryCallbook As Boolean, UpdateeQSLMembership As Boolean, UpdateLoTWMembership As Boolean, CheckOverrides As Boolean) As Boolean
    Dim theParams As String
    Dim theResult As String
    
    '1.4.3
        
    On Error GoTo Abort
    
    ExternalLog = False
    
    If Not DXLabControlMain.LogTCPCmd.Enabled Then Exit Function
    
    DXLabControlMain.LogTCPCmd.Enabled = False

    theParams = ""
    theParams = theParams + DDECommon.EncodeField(ExternalLogADIF, theADIF)
    theParams = theParams + DDECommon.EncodeField(ExternalLogUploadeQSL, DDEClientModule.BooleanParamString(UploadeQSL))
    theParams = theParams + DDECommon.EncodeField(ExternalLogUploadLoTW, DDEClientModule.BooleanParamString(UploadLoTW))
    theParams = theParams + DDECommon.EncodeField(ExternalLogDeduceMissing, DDEClientModule.BooleanParamString(DeduceMissing))
    theParams = theParams + DDECommon.EncodeField(ExternalLogQueryCallbook, DDEClientModule.BooleanParamString(QueryCallbook))
    theParams = theParams + DDECommon.EncodeField(ExternalLogQueryCallbook, DDEClientModule.BooleanParamString(UpdateeQSLMembership))
    theParams = theParams + DDECommon.EncodeField(ExternalLogQueryCallbook, DDEClientModule.BooleanParamString(UpdateLoTWMembership))
    theParams = theParams + DDECommon.EncodeField(ExternalLogQueryCallbook, DDEClientModule.BooleanParamString(CheckOverrides))
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, DXKeeperTCPOffset, ExternalLogCommand, theParams)
    Select Case LCase$(theResult)
    
        Case "sent"
            ExternalLog = True
            
        Case "busy"
            ExternalLog = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "Log via TCP/IP"
            ExternalLog = False
            
    End Select

    DXLabControlMain.LogTCPCmd.Enabled = True

    Exit Function

Abort:

    Common.ProgramError Err.Number, "TCPClientModule.ExternalLog"

End Function


Function Delete(theRecord As String) As Boolean
    
    Dim theResult As String
    
    On Error GoTo Abort
    
    '1.3.8
    Delete = False
    
    If Not DXLabControlMain.DeleteTCPCmd.Enabled Then Exit Function
    
    DXLabControlMain.DeleteTCPCmd.Enabled = False

    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, DXKeeperTCPOffset, DeleteQSOCommand, theRecord)
    Select Case LCase$(theResult)
    
        Case "sent"
            Delete = True
            
        Case "busy"
            Delete = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "Delete QSO via TCP/IP"
            Delete = False
            
    End Select

    DXLabControlMain.DeleteTCPCmd.Enabled = True
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.Delete"
    
End Function

Function UpdateXCVRFreqMode(theFreq As String, theMode As String, PreserveDualandSplit As Boolean) As Boolean

    Dim theRecord As String
    
    Dim theResult As String
    
    Dim Preservation As String
    
    On Error GoTo Abort
    
    UpdateXCVRFreqMode = False
    
    If Not DXLabControlMain.TCPUpdateCmd.Enabled Then Exit Function
    
    DXLabControlMain.TCPUpdateCmd.Enabled = False
    
    If PreserveDualandSplit Then
        Preservation = "Y"
    Else
        Preservation = "N"
    End If
    
    theRecord = DDECommon.EncodeField(XcvrFreq, theFreq) + DDECommon.EncodeField(XcvrMode, theMode) + DDECommon.EncodeField(XcvrPreserveSplitAndDual, Preservation)
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SetFreqModeCmd, theRecord, True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
            UpdateXCVRFreqMode = True
            
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "UpdateXCVRFreqMode"
            UpdateXCVRFreqMode = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "UpdateXCVRFreqMode"
            UpdateXCVRFreqMode = False
            
    End Select

    DXLabControlMain.TCPUpdateCmd.Enabled = True
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.UpdateXCVRFreqMode"
    
End Function

Function UpdateQSXSplit(theFreq As String, SuppressSplit As Boolean) As Boolean '1.3.6
    
    Dim theRecord As String
    
    Dim theResult As String
    
    Dim Suppression As String
    
   
    On Error GoTo Abort
    
    UpdateQSXSplit = False
    
    If Not DXLabControlMain.TCPUpdateCmd.Enabled Then Exit Function
    
    DXLabControlMain.TCPQSXSplitCmd.Enabled = False
    
    If SuppressSplit Then
        Suppression = "Y"
    Else
        Suppression = "N"
    End If
    
    
    theRecord = DDECommon.EncodeField(XcvrFreq, theFreq) + DDECommon.EncodeField("SuppressDual", Suppression)
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SetQSXSplitCmd, theRecord)
    
    Select Case LCase$(theResult)
    
        Case "sent"
            UpdateQSXSplit = True
            
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "UpdateQSXSplit"
            UpdateQSXSplit = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "UpdateQSXSplit"
            UpdateQSXSplit = False
            
    End Select

    DXLabControlMain.TCPQSXSplitCmd.Enabled = True
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.UpdateQSXSplit"
End Function

Function UpdateRXFreq(theFreq As String) As Boolean

    Dim theRecord As String
    
    Dim theResult As String
       
    On Error GoTo Abort
    
    UpdateRXFreq = False
    
    If Not DXLabControlMain.TCPRXFreqUpdateCmd.Enabled Then Exit Function
    
    DXLabControlMain.TCPRXFreqUpdateCmd.Enabled = False
    
    theRecord = DDECommon.EncodeField(XcvrFreq, theFreq)
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SetFreqCmd, theRecord, True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
            UpdateRXFreq = True
            
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "UpdateRXFreq"
            UpdateRXFreq = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "UpdateRXFreq"
            UpdateRXFreq = False
            
    End Select

    DXLabControlMain.TCPRXFreqUpdateCmd.Enabled = True
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.UpdateRXFreq"
    
End Function


Function UpdateTXFreq(theFreq As String) As Boolean

    Dim theRecord As String
    
    Dim theResult As String
       
    On Error GoTo Abort
    
    UpdateTXFreq = False
    
    If Not DXLabControlMain.TCPTXFreqUpdateCmd.Enabled Then Exit Function
    
    DXLabControlMain.TCPTXFreqUpdateCmd.Enabled = False
    
    theRecord = DDECommon.EncodeField(XcvrFreq, theFreq)
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SetTXFreqCmd, theRecord, True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
            UpdateTXFreq = True
            
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "UpdateTXFreq"
            UpdateTXFreq = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "UpdateTXFreq"
            UpdateTXFreq = False
            
    End Select

    DXLabControlMain.TCPTXFreqUpdateCmd.Enabled = True
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.UpdateTXFreq"
    
End Function

Function UpdateMode(theMode As String) As Boolean

    Dim theRecord As String
    Dim theResult As String
       
    On Error GoTo Abort
    
    UpdateMode = False
    
    If Not DXLabControlMain.TCPModeUpdateCmd.Enabled Then Exit Function
    
    DXLabControlMain.TCPModeUpdateCmd.Enabled = False
    
    theRecord = DDECommon.EncodeField(1, theMode)
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SetModeCmd, theRecord, True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
            UpdateMode = True
            
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "UpdateTXFreq"
            UpdateMode = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "UpdateTXFreq"
            UpdateMode = False
            
    End Select

    DXLabControlMain.TCPModeUpdateCmd.Enabled = True
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.UpdateMode"
    
End Function

Function UpdateVFO(theVFO As Integer) As Boolean

    Dim theRecord As String
    Dim theResult As String
       
    On Error GoTo Abort
    
    UpdateVFO = False
    
    If Not DXLabControlMain.TCPVFOUpdateCmd.Enabled Then Exit Function
    
    DXLabControlMain.TCPVFOUpdateCmd.Enabled = False
    
    theRecord = DDECommon.EncodeField(1, Format$(theVFO))
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SelectVFOCmd, theRecord, True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
            UpdateVFO = True
            
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "UpdateTXFreq"
            UpdateVFO = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "UpdateTXFreq"
            UpdateVFO = False
            
    End Select

    DXLabControlMain.TCPVFOUpdateCmd.Enabled = True
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.UpdateVFO"
    
End Function

Function SetSplit(theValue As Boolean) As Boolean
    
    Dim Arg As String
    
    Dim theRecord As String
    Dim theResult As String
    
    On Error GoTo Abort
    
    SetSplit = False
    
    If theValue Then
        Arg = "on"
    Else
        Arg = "off"
    End If
    
    theRecord = DDECommon.EncodeField(1, Arg)
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SetSplitCmd, theRecord, True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
            SetSplit = True
            
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "SetSplit"
            SetSplit = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "SetSplit"
           SetSplit = False
            
    End Select

    Exit Function
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.SetSplit"
    
End Function

Function SetTransmit() As Boolean
    
    Dim theResult As String
    
    On Error GoTo Abort
    
    SetTransmit = False
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SetTXCmd, "", True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
            SetTransmit = True
            
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "SetTransmit"
            SetTransmit = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "SetTransmit"
           SetTransmit = False
            
    End Select

    Exit Function
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.SetTransmit"
    
End Function

Function SetReceive() As Boolean
    
    Dim theResult As String
    
    On Error GoTo Abort
    
    SetReceive = False
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SetRXCmd, "", True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
            SetReceive = True
            
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "SetReceive"
            SetReceive = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "SetReceive"
           SetReceive = False
            
    End Select

    Exit Function
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.SetReceive"
    
End Function

Sub RequestRXVFO()
    
    Dim theResult As String
     
    On Error GoTo Abort
    
    DXLabControlMain.TCPUpdateRXFreq.Enabled = False
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SendFreqCmd, "", True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
                        
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "RequestRXVFO"
            DXLabControlMain.TCPUpdateRXFreq.Enabled = True
            Exit Sub
            
        Case Else
            MsgBox theResult, vbOKOnly, "RequestRXVFO"
            DXLabControlMain.TCPUpdateRXFreq.Enabled = True
            Exit Sub
            
    End Select
    
    DXLabControlMain.TCPUpdateRXFreq.Enabled = True
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.RequestRXVFO"
End Sub

Sub RequestTXVFO()
    
    Dim theResult As String
     
    On Error GoTo Abort
    
    DXLabControlMain.TCPUpdateTXFreq.Enabled = False
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SendTXFreqCmd, "", True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
                        
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "RequestTXVFO"
            DXLabControlMain.TCPUpdateTXFreq.Enabled = True
            Exit Sub
            
        Case Else
            MsgBox theResult, vbOKOnly, "RequestTXVFO"
            DXLabControlMain.TCPUpdateTXFreq.Enabled = True
            Exit Sub
            
    End Select
    
    DXLabControlMain.TCPUpdateTXFreq.Enabled = True
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.RequestTXVFO"
End Sub

Function SyncIcom() As Boolean
    
    Dim theResult As String
    
    On Error GoTo Abort
    
    SyncIcom = False
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SyncIcomCmd, "", True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
            SyncIcom = True
            
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "SyncIcom"
            SyncIcom = False
            
        Case Else
            MsgBox theResult, vbOKOnly, "SyncIcom"
           SyncIcom = False
            
    End Select

    Exit Function
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.SyncIcom"
    
End Function
Sub RequestMode()
    
    Dim theResult As String
    
    On Error GoTo Abort
    
    DXLabControlMain.TCPUpdateMode.Enabled = False
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SendModeCmd, "", True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
                        
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "RequestMode"
            DXLabControlMain.TCPUpdateMode.Enabled = True
            Exit Sub
            
        Case Else
            MsgBox theResult, vbOKOnly, "RequestMode"
            DXLabControlMain.TCPUpdateMode.Enabled = True
            Exit Sub
            
    End Select
    
    DXLabControlMain.TCPUpdateMode.Enabled = True
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.RequestMode"
End Sub

Sub RequestSplit()
    
    Dim theResult As String
    
    On Error GoTo Abort
    
    DXLabControlMain.TCPUpdateSplit.Enabled = False
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SendSplitCmd, "", True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
                        
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "RequestSplit"
            DXLabControlMain.TCPUpdateSplit.Enabled = True
            Exit Sub
            
        Case Else
            MsgBox theResult, vbOKOnly, "RequestSplit"
            DXLabControlMain.TCPUpdateSplit.Enabled = True
            Exit Sub
            
    End Select
    
    DXLabControlMain.TCPUpdateSplit.Enabled = True
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.RequestSplit"
End Sub

Sub RequestTX()
    
    Dim theResult As String
    
    On Error GoTo Abort
    
    DXLabControlMain.TCPUpdateTX.Enabled = False
    
    theResult = TCPClientModule.SendCommand(TCPClient.TCPServerAddress, CommanderTCPOffset, SendTXCmd, "", True)
    
    Select Case LCase$(theResult)
    
        Case "sent"
                        
        Case "busy"
            MsgBox "not accepted", vbOKOnly, "RequestSplit"
            DXLabControlMain.TCPUpdateTX.Enabled = True
            Exit Sub
            
        Case Else
            MsgBox theResult, vbOKOnly, "RequestSplit"
            DXLabControlMain.TCPUpdateTX.Enabled = True
            Exit Sub
            
    End Select
    
    DXLabControlMain.TCPUpdateTX.Enabled = True
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.RequestTX"
End Sub

Sub UDPSocketError(Number As Integer, Description As String)

    On Error GoTo Abort
    
    Debug.Print Description
    
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.UDPSocketError"
    
End Sub

Function ExtractXMLField(theRecord As String, theField As String) As String
    On Error GoTo Abort
    
    Dim Pos1 As Integer
    Dim Pos2 As Integer
    Dim StartTag As String
    
    StartTag = "<" + theField + ">"
    ExtractXMLField = ""
    
    Pos1 = InStr(1, theRecord, StartTag)
    If Pos1 > 0 Then
        Pos2 = InStr(Pos1 + Len(StartTag), theRecord, "</" + theField + ">")
        If Pos2 > 0 Then
            ExtractXMLField = Mid$(theRecord, Pos1 + Len(StartTag), Pos2 - Pos1 - Len(StartTag))
        End If
    End If
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.ExtractXMLField"

End Function

Sub UDPDataArrival(ByteCount As Long)

    Dim theRecord As String
    Dim UpperRecord As String
    Dim Freq As Double
    Dim TXFreq As Double
    Dim Mode As String

    
    On Error GoTo Abort
    
    TCPClient.UDPSocket.GetData theRecord, vbString
    UpperRecord = UCase$(theRecord)
    
    UpperRecord = TCPClientModule.ExtractXMLField(UpperRecord, "RADIOINFO")

    Freq = CDbl(TCPClientModule.ExtractXMLField(UpperRecord, "FREQ"))
    TXFreq = CDbl(TCPClientModule.ExtractXMLField(UpperRecord, "TXFREQ"))
    Mode = TCPClientModule.ExtractXMLField(UpperRecord, "MODE")
    
    DXLabControlMain.TCPXcvrVFO.Caption = Format$(Freq / 100, "###,###,##0.00")
    DXLabControlMain.TCPXcvrTXVFO.Caption = Format$(TXFreq / 100, "###,###,##0.00")
    DXLabControlMain.TCPXcvrRadioMode = Mode
    
    Exit Sub
    
Abort:

    Common.ProgramError Err.Number, "TCPClientModule.UDPDataArrived"
    
End Sub
