Attribute VB_Name = "TCPCommon"
Option Explicit

Global Const DefaultTCPAddress = "127.0.0.1"
Global Const DefaultTCPBasePort = "52000"

Global Const DefaultUDPAddress = "127.0.0.1"
Global Const DefaultUDPPort = "12060"

'TCPPortOffsets
Global Const DXKeeperTCPOffset = 1
Global Const CommanderTCPOffset = 2
Global Const MaxTCPOffset = 7

'Commander directives
Global Const SetFreqModeCmd = "CmdSetFreqMode"
Global Const SetQSXSplitCmd = "CmdQSXSplit" '10.9.9
Global Const SetFreqCmd = "CmdSetFreq"
Global Const SetTXFreqCmd = "CmdSetTXFreq"
Global Const SetModeCmd = "CmdSetMode"
Global Const SetTXCmd = "CmdTX"
Global Const SetRXCmd = "CmdRX"
Global Const SetSplitCmd = "CmdSplit"
Global Const GetFreqCmd = "CmdGetFreq"
Global Const GetTXFreqCmd = "CmdGetTXFreq"
Global Const GetModeCmd = "CmdGetMode"
Global Const SendFreqCmd = "CmdSendFreq"
Global Const SendTXFreqCmd = "CmdSendTXFreq"
Global Const SendModeCmd = "CmdSendMode"
Global Const SendSplitCmd = "CmdSendSplit"
Global Const SendTXCmd = "CmdSendTX"
Global Const SelectVFOCmd = "CmdSelectVFO"
Global Const SyncIcomCmd = "CmdSyncIcom"

'Commander responses
Global Const FreqResp = "CmdFreq"
Global Const TXFreqResp = "CmdTXFreq"
Global Const ModeResp = "CmdMode"
Global Const SplitResp = "CmdSplit"
Global Const TXResp = "CmdTX"

Type ParsedMessageKind
    Command As String
    Parameters As String
    Remainder As String
End Type

Dim LastParsedValueEndPos As Integer

Const EndOfTCPParameters = "<eor>"

Const MaxTCPParameters = 100
    
Type TCPParameter
    Name As String
    Content As String
End Type

Type TCPParseResult
    ParameterCount As Integer
    ParameterArray(1 To MaxTCPParameters) As TCPParameter
End Type

Function WinsockStatus(theState As Integer) As String

    On Error GoTo Abort
    
    Select Case theState
    
        Case sckClosed
            WinsockStatus = "Closed"
            
        Case sckOpen
            WinsockStatus = "Open"
        
        Case sckListening
            WinsockStatus = "Listening"
        
        Case sckConnectionPending
            WinsockStatus = "Connection pending"
        
        Case sckResolvingHost
            WinsockStatus = "Resolving host"
        
        Case sckHostResolved
            WinsockStatus = "Host resolved"
        
        Case sckConnecting
            WinsockStatus = "Connecting"
        
        Case sckConnected
            WinsockStatus = "Connected"
        
        Case sckClosing
            WinsockStatus = "Peer is closing the connection"
            
        Case sckError
            WinsockStatus = "Error"
        
        Case Else
            WinsockStatus = "unknown: " + Format$(theState)
  
    End Select
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPCommon.WinsockStatusString"
    
End Function
        
Function TagValue(Message As String, Tag As String) As String

    Dim TagStartPos As Integer
    Dim ColonPos As Integer
    Dim TagEndPos As Integer
    Dim Length As Long
    Dim LowerMessage As String
    
    On Error GoTo Abort
    
    TagValue = ""
    LastParsedValueEndPos = -1
    
    LowerMessage = LCase$(Message)
    
    TagStartPos = InStr(LowerMessage, Tag)
    If TagStartPos = 0 Then
        Exit Function
    End If
    
    ColonPos = InStr(TagStartPos + Len(Tag), LowerMessage, ":")
    If ColonPos = 0 Then
        Exit Function
    End If
        
    TagEndPos = InStr(TagStartPos + Len(Tag), LowerMessage, ">")
    If TagEndPos = 0 Then
        Exit Function
    ElseIf TagEndPos < ColonPos Then
        Exit Function
    End If
    
    Length = -1
    On Error Resume Next
    Length = CLng(Mid$(LowerMessage, ColonPos + 1, TagEndPos - ColonPos - 1))
    On Error GoTo Abort
    
    LastParsedValueEndPos = TagEndPos + Length '11.4.0
    
    If Length <= 1 Then
        Exit Function
    End If
    
    On Error Resume Next 'in case supplied length is wrong
    TagValue = Mid$(Message, TagEndPos + 1, Length)
    
    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPCommon.TagValue"
End Function

Function ParseMessage(Message As String) As ParsedMessageKind
    
    Const CmdTag = "<command"
    Const ParamTag = "<parameters"
    
    On Error GoTo Abort

    ParseMessage.Command = TCPCommon.TagValue(Message, CmdTag)
    ParseMessage.Parameters = TCPCommon.TagValue(Message, ParamTag)
    If (Len(Message) <= LastParsedValueEndPos) Or (LastParsedValueEndPos = -1) Then
        ParseMessage.Remainder = ""
    Else
        ParseMessage.Remainder = Mid$(Message, LastParsedValueEndPos + 1, Len(Message) - LastParsedValueEndPos)
    End If

    Exit Function
    
Abort:

    Common.ProgramError Err.Number, "TCPCommon.ParseMessage"
    
End Function

Function EncodeParameter(Name As String, Data As String) As String
    Dim theLength As Integer
    
    On Error GoTo Abort
    
    EncodeParameter = ""
    
    If Len(Name) = 0 Then Exit Function
    
    theLength = Len(Data)
    EncodeParameter = "<" + Name + ":" + Format$(theLength) + ">" + Data
    
    Exit Function

Abort: Common.ProgramError Err.Number, "TCPCommon.EncodeParameter"

End Function

Function ParseParameters(EncodedData As String) As TCPParseResult

    Dim FieldDefStart As Integer
    Dim FieldDefEnd As Integer
    Dim FieldDef As String
    Dim FieldNameEnd As Integer
    Dim Fieldname As String
    Dim FieldLength As String
    Dim FieldLengthValue As Integer
    Dim FieldContent As String
    
    Dim theResult As TCPParseResult
        
    Dim theData As String
    
    On Error GoTo Abort

    theData = EncodedData
    theResult.ParameterCount = 0
    
    While Len(theData) > 0
    
        FieldDefStart = InStr(theData, "<")
        
        If FieldDefStart = 0 Then
            theData = "" 'ignore whatever is left
            
        ElseIf FieldDefStart > 0 Then
            If LCase$(Mid$(theData, FieldDefStart, Len(EndOfTCPParameters))) = EndOfTCPParameters Then
                theData = ""
            End If
            
            FieldDefEnd = InStr(FieldDefStart + 1, theData, ">")
            If FieldDefEnd = 0 Then
                theData = "" 'ignore whatever is left
            
            ElseIf FieldDefEnd > 0 Then
                FieldDef = Mid$(theData, FieldDefStart + 1, FieldDefEnd - FieldDefStart - 1)
                FieldNameEnd = InStr(FieldDef, ":")
                If FieldNameEnd = 0 Then
                    theData = "" 'ignore whatever is left
                
                ElseIf FieldNameEnd > 0 Then
                    Fieldname = Mid$(FieldDef, 1, FieldNameEnd - 1)
                    FieldLength = Mid$(FieldDef, FieldNameEnd + 1, Len(FieldDef) - FieldNameEnd)
                    
                    FieldLengthValue = 0
                    On Error Resume Next
                    FieldLengthValue = CInt(FieldLength)
                    On Error GoTo Abort
                    
                    FieldContent = Mid$(theData, FieldDefEnd + 1, FieldLengthValue)
                       
                    theResult.ParameterCount = theResult.ParameterCount + 1
                    theResult.ParameterArray(theResult.ParameterCount).Name = Fieldname
                    theResult.ParameterArray(theResult.ParameterCount).Content = FieldContent
                    
                    '9.5.5 tolerate incorrect length specifier
                    If (Len(theData) - FieldDefEnd - FieldLengthValue) > 0 Then
                        theData = Mid$(theData, FieldDefEnd + FieldLengthValue + 1, Len(theData) - FieldDefEnd - FieldLengthValue)
                    Else
                        theData = "" 'terminate
                    End If
                    
                    'Debug.Print Fieldname; " "; FieldContent
                  
                End If
            
            End If
        
        End If
        
    Wend

    ParseParameters = theResult
    
    Exit Function
    
Abort:
    Common.ProgramError Err.Number, "TCPCommon.ParseParameters, EncodedData = " + EncodedData
End Function



